unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, VirtualTrees, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ImgList;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnGo: TButton;
    FileSystemBitmaps: TImageList;
    MyStringTree: TVirtualStringTree;
    procedure btnGoClick(Sender: TObject);
    procedure MyStringTreeFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode
      );
    procedure MyStringTreeGetImageIndexEx(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer;
      var ImageList: TCustomImageList);
    procedure MyStringTreeGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure MyStringTreeGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
  private

    procedure AddDirectoryContentsAsChildNodes(const ParentDirPath: string; Node: PVirtualNode);

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.frm}

type

  { TFilesystemNode }

  // TODO: use a record instead of a class ?

  TFilesystemNode = class
  private
    Name: string;
    Attr: Longint;
  public
    constructor Create(Name_: string; Attr_: Longint);
  end;

{ TFilesystemNode }

constructor TFilesystemNode.Create(Name_: string; Attr_: Longint);
begin
  Name := Name_;
  Attr := Attr_;
end;

{ TForm1 }

procedure TForm1.MyStringTreeGetNodeDataSize(Sender: TBaseVirtualTree;
  var NodeDataSize: Integer);
begin
  NodeDataSize := sizeof(TFilesystemNode);
end;

procedure TForm1.MyStringTreeGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var
  FSNode: TFilesystemNode;
begin
  FSNode := TFilesystemNode(Sender.GetNodeData(Node)^);
  CellText := FSNode.Name;
end;

procedure TForm1.AddDirectoryContentsAsChildNodes(const ParentDirPath: string;
  Node: PVirtualNode);
var
  SearchPath: string;
  sr: TSearchRec;
  data: ^TFilesystemNode;
begin
  SearchPath := ParentDirPath + '\*.*'; // because Windows
  if FindFirst(SearchPath, faAnyFile, sr) = 0 then
  begin
    repeat
      data := MyStringTree.GetNodeData(MyStringTree.AddChild(Node));
      data^ := TFilesystemNode.Create(sr.Name, sr.Attr);
    until FindNext(sr) <> 0;
    FindClose(sr);
  end;
end;

procedure TForm1.btnGoClick(Sender: TObject);
begin
  MyStringTree.BeginUpdate;
  try
    // TODO: use filesystem root(s) as appropriate for the operating system
    AddDirectoryContentsAsChildNodes('C:/', MyStringTree.RootNode);
  finally
    MyStringTree.EndUpdate;
  end;
end;

procedure TForm1.MyStringTreeFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  FSNode: ^TFilesystemNode;
begin
  FSNode := MyStringTree.GetNodeData(Node);
  FreeAndNil(FSNode^);
end;

procedure TForm1.MyStringTreeGetImageIndexEx(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer; var ImageList: TCustomImageList
  );
var
  FSNode: ^TFilesystemNode;
  Attr: Longint;
begin
  FSNode := MyStringTree.GetNodeData(Node);
  Attr := FSNode^.Attr;
  if Attr and faDirectory <> 0 then
    ImageIndex := 0
  else
    ImageIndex := 1;
  if Attr and faHidden <> 0 then
    Ghosted := true;
end;

end.

